'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

// hooking Middleware

var hooks = exports.hooks = {};

/**
 * register dispatch hook
 * @param  {'pre'|'post'} position define pre or post hook
 * @param  {string}       type     action type
 * @param  {function}     hook     hook
 * @return {false|symbol}          return hook id or false when failed
 */
var registerHook = exports.registerHook = function registerHook(position, type, hook) {
  if (typeof position !== 'string') {
    return false;
  }
  if (typeof type !== 'string') {
    return false;
  }
  if (typeof hook !== 'function') {
    return false;
  }

  if (!hooks[type]) {
    hooks[type] = [];
  }

  var id = Symbol(type);
  hooks[type].push({ position: position, id: id, hook: hook });

  return id;
};

/**
 * curried hooking function
 * @param  {string}   type action type
 * @param  {function} hook hook
 * @return {function}      curried register function
 */
var registerPrehook = exports.registerPrehook = function registerPrehook(type, hook) {
  return registerHook('pre', type, hook);
};

/**
* curried hooking function
 * @param  {string}   type action type
 * @param  {function} hook hook
 * @return {function}      curried register function
 */
var registerPosthook = exports.registerPosthook = function registerPosthook(type, hook) {
  return registerHook('post', type, hook);
};

/**
 * registerPrehooks
 * @param  {'pre'|'post'} position define pre or post hook
 * @param  {object} structTypes type: <Array<Function>> or <Function>
 * @return {object}             ids
 */
var registerHooks = exports.registerHooks = function registerHooks(position, structTypes) {

  if (typeof position !== 'string') {
    return false;
  }
  if ((typeof structTypes === 'undefined' ? 'undefined' : _typeof(structTypes)) !== 'object') {
    return false;
  }

  var result = {};

  Object.keys(structTypes).forEach(function (type) {
    var hooks = structTypes[type];
    if (typeof hooks === 'function') {
      result[type] = [registerHook(position, type, hooks)];
    } else if (Array.isArray(hooks)) {
      result[type] = hooks.map(function (hook) {
        return typeof hook === 'function' ? registerHook(position, type, hook) : void 0;
      });
    } else {
      result[type] = void 0;
    }
  });

  return result;
};

/**
 * register Prehooks
 * @param  {object} structTypes type: <Array<Function>> or <Function>
 * @return {object}             ids
 */
var registerPrehooks = exports.registerPrehooks = function registerPrehooks(structTypes) {
  return registerHooks('pre', structTypes);
};

/**
 * register Prehooks
 * @param  {object} structTypes type: <Array<Function>> or <Function>
 * @return {object}             ids
 */
var registerPosthooks = exports.registerPosthooks = function registerPosthooks(structTypes) {
  return registerHooks('post', structTypes);
};
/**
 * unregister dispatch hook
 * @param  {symbol} id given hook
 * @return {void}
 */
var unregisterHook = exports.unregisterHook = function unregisterHook(id) {
  Object.keys(hooks).forEach(function (type) {
    hooks[type] = hooks[type].filter(function (x) {
      return x.id !== id;
    });
  });
};

/**
 * clear all hooks
 * @return {void}
 */
var clearHooks = exports.clearHooks = function clearHooks() {
  Object.keys(hooks).forEach(function (type) {
    delete hooks[type];
  });
};

/**
 * Redux Middleware to chain api call
 * @param  {Store}      store  Redux store
 * @param  {Middleware} next   next middlwware
 * @param  {Action}     action dispatched action
 * @return {void}
 */

exports.default = function (store) {
  return function (next) {
    return function (action) {
      /**
       * [getTheHooksAt description]
       * @param  {'pre'|'post'}     position determine pre or post
       * @return {array<function>}           hooks
       */
      var getTheHooksAt = function getTheHooksAt(position) {
        return action && hooks[action.type] ? hooks[action.type].filter(function (x) {
          return x.position === position;
        }).map(function (x) {
          return x.hook;
        }) : [];
      };
      var prehooks = getTheHooksAt('pre');
      var posthooks = getTheHooksAt('post');

      prehooks.forEach(function (hook) {
        return hook(store, action);
      });
      var result = next(action);
      posthooks.forEach(function (hook) {
        return hook(store, action);
      });
      return result;
    };
  };
};
//# sourceMappingURL=index.js.map